﻿
Module Module_Utils

    ' =======================================================================================================
    '   Platform selection
    ' =======================================================================================================

    Friend CanUseWindowsDlls As Boolean = _CanUseWindowsDlls()

    Friend Function _CanUseWindowsDlls() As Boolean
        Select Case Environment.OSVersion.Platform
            Case PlatformID.Win32NT, PlatformID.Win32S, PlatformID.Win32Windows, PlatformID.WinCE
                Return True
            Case Else
                Return False
        End Select
    End Function

    Friend Function PlatformAdjustedFileName(ByVal FileName As String, _
                                         Optional ByVal DefaultName As String = "") As String
        If CanUseWindowsDlls Then
            FileName = Replace(FileName, "/", "\")
            Return FileName
        Else
            If FileName.Contains(":") Then FileName = DefaultName
            FileName = Replace(FileName, "\", "/")
            Return FileName
        End If
    End Function


    ' =======================================================================================
    '  LimitFormPosition
    ' =======================================================================================

    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
    End Sub

    Private sw As Integer
    Private sh As Integer
    Private Sub GetMaxScreenBounds()
        ' -------------------------------------- Gets an array of all the screens connected to the system.
        Dim Screens() As System.Windows.Forms.Screen = System.Windows.Forms.Screen.AllScreens
        ' -------------------------------------- For each screen
        Dim Index As Integer
        For Index = 0 To Screens.GetUpperBound(0)
            sw = Math.Max(sw, Screens(Index).Bounds.Left + Screens(Index).Bounds.Width)
            sh = Math.Max(sh, Screens(Index).Bounds.Top + Screens(Index).Bounds.Height)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, sw)           ' not more than a maximized window
    '        .Height = Math.Min(.Height, sh)         ' not more than a maximized window
    '        .Width = Math.Max(.Width, 32)           ' at least 32x24
    '        .Height = Math.Max(.Height, 24)         ' at least 32x24
    '        .Left = Math.Min(.Left, sw - .Width)    ' not beyond the right border
    '        .Top = Math.Min(.Top, sh - .Height)     ' not beyond the bottom border
    '        .Left = Math.Max(.Left, 0)              ' at least at the left border
    '        .Top = Math.Max(.Top, 0)                ' at least at the top border
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, sw)               ' not more than the VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, sh)             ' not more than the VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)               ' at least 32x24
            .Height = Math.Max(.Height, 24)             ' at least 32x24
            .Left = Math.Min(.Left, sw - 50)            ' not beyond the right border - 50 pixels
            .Top = Math.Min(.Top, sh - 50)              ' not beyond the bottom border - 50 pixels
            .Left = Math.Max(.Left, 50 - .Width)        ' at least at the left border + 50 pixels
            .Top = Math.Max(.Top, -30)                  ' at least at the top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' =======================================================================================
    '  Utils
    ' =======================================================================================

    Friend Sub SleepMyThread(ByVal TimeMillisec As Int32)
        System.Threading.Thread.Sleep(TimeMillisec)
    End Sub

End Module



' =============================================================================================
'  CLASS ReverseIterator
' =============================================================================================

Friend Class ReverseIterator
    Implements IEnumerable
    ' ------------------------------------------ a low-overhead ArrayList to store references
    Dim items As New ArrayList()

    Sub New(ByVal collection As IEnumerable)
        ' -------------------------------------- load all the items in the ArrayList, but in reverse order
        Dim o As Object
        For Each o In collection
            items.Insert(0, o)
        Next
    End Sub
    '
    Public Function GetEnumerator() As System.Collections.IEnumerator _
        Implements System.Collections.IEnumerable.GetEnumerator
        ' ------------------------------------- return the enumerator of the inner ArrayList
        Return items.GetEnumerator()
    End Function
End Class

